#!/usr/bin/env python3
# vim: set ft=python fileencoding=utf-8 tw=72:
r"""
A blog-oriented static web content generator.

* Input formats: Markdown and reStructuredText
* Output format: HTML5.

:author: \J. A. Corbal
:email: jacorbal@gmail.com
:copyright: © 2012-2020, J. A. Corbal
:license: MIT
"""

from pynfact.main import main


if __name__ == '__main__':
    main()
